/*
 *    HardInfo - Displays System Information
 *    Copyright (C) 2003-2006 Leandro A. F. Pereira <leandro@hardinfo.org>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, version 2.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */

struct _Processor {
	gint Core_Count;
	gint cpu_part;
	gint processor;
	gint Speed;
	gint cpu_implementer;
	gint Version;
	gchar * Family;
	gchar * Manufacturer;
	gfloat cpu_mhz;
	gchar * model_name;
	gchar * Core_Enabled;
	gchar * External_Clock;
	gchar * Current_Speed;
	gchar * ID;        
	gchar * Thread_Count;
	gchar * Voltage;
	gchar * Threa_Count;
	gchar * Byte_Order;
	gchar * Byte_Order_zh;
	gchar * BogoMIPS;
	gchar * L1d;
	gchar * L1i;
	gchar * L2;
	gchar * L3;
	gchar * mode;
};

static GSList *__scan_processors(void)
{
    system("get_disk_info.sh");
    Processor *processor = NULL;
    FILE *cpuinfo;
    gchar buffer[256];

    cpuinfo = fopen("/tmp/cpuinfo", "r");
    if (!cpuinfo)
	return NULL;

    processor = g_new0(Processor,1);
    while (fgets(buffer, 256, cpuinfo)) {
	gchar **tmp = g_strsplit(buffer, ":", 2);

	if (tmp[0] && tmp[1]) {
	    tmp[0] = g_strstrip(tmp[0]);
	    tmp[1] = g_strstrip(tmp[1]);
	    get_str("Family", processor->Family);
	    get_str("Manufacturer", processor->Manufacturer);
	    get_str("Version", processor->Version);
	    get_str("Core Count", processor->Core_Count);
	    get_int("CPU part", processor->cpu_part);
	    get_str("Max Speed",processor->Speed);
	    get_float("spend_max",processor->cpu_mhz);  
	    get_str("Core Enabled",processor->Core_Enabled);
	    get_str("External Clock",processor->External_Clock);
	    get_str("Current Speed",processor->Current_Speed);
	    get_str("ID",processor->ID);
	    get_str("Voltage",processor->Voltage);
	    get_str("Thread Count",processor->Thread_Count);
	    get_str("Byte Order",processor->Byte_Order);
	    get_str("字节序",processor->Byte_Order_zh);  
	    get_str("BogoMIPS",processor->BogoMIPS);
	    get_str("L1d",processor->L1d);
	    get_str("L1i",processor->L1i);
	    get_str("L2",processor->L2);
	    get_str("L3",processor->L3);
	    get_str("mode",processor->mode);
	}
	g_strfreev(tmp);
    }
    
    	
	
          	
    processor->model_name = processor->Version;

    fclose(cpuinfo);

    return g_slist_append(NULL,processor);
}


static gchar *processor_get_info(GSList * processors)
{
    Processor *processor = (Processor *)processors->data;
    if((0 == strcmp(getenv("LANG"), "zh_CN.UTF-8")) || (0 == strcmp(getenv("LANG"), "zh_CN.utf8")))
  {
        return g_strdup_printf("[Processor]\n"
                        "系列=%s\n"
                        "制造商=%s\n"
                        "型号=%s\n"
                        "最高速度=%s\n"
                        "当前速度=%s\n"
                        "外部时钟=%s\n"
			"核心总数=%s\n"
			"核心启用=%s\n"
			"线程数=%s\n"
			"电压=%s\n"
			"字节序=%s\n"
			"BogoMIPS=%s\n"
			"L1d 缓存=%s\n"
			"L1i 缓存=%s\n"
			"L2  缓存=%s\n"
			"L3  缓存=%s\n"
			"运行模式=%s位\n",
                        processor->Family,
                        processor->Manufacturer,
                        processor->Version,
                        processor->Speed,
                        processor->Current_Speed,
                        processor->External_Clock,
 			processor->Core_Count,
 			processor->Core_Enabled,
 			processor->Thread_Count,
 			processor->Voltage,
 			processor->Byte_Order,
 			processor->BogoMIPS!=NULL? processor->BogoMIPS:"UnKnown",
 			processor->L1d!=NULL ? processor->L1d : "Not Provided",
 			processor->L1i!=NULL ? processor->L1i : "Not Provided",
 			processor->L2,
 			processor->L3,
 			processor->mode
                        );
  }else{
	    return g_strdup_printf("[Processor]\n"
                        "Family=%s\n"
                        "Manufacturer=%s\n"
                        "Version=%s\n"          
                        "Max Speed=%s\n"
			"Current Speed=%s\n"
                        "External Clock=%s\n"
                        "Core Count=%s\n"
                        "Core Enabled=%s\n"
                        "Thread Count=%s\n"
                        "Voltage=%s\n"
			"Byte Order=%s\n"
			"BogoMIPS=%s\n"
			"L1d cache=%s\n"
			"L1i cache=%s\n"
			"L2  cache=%s\n"
			"L3  cache=%s\n"
			"mode=%s\n",
                        processor->Family,
                        processor->Manufacturer,
                        processor->Version,
                        processor->Speed,
			processor->Current_Speed,
                        processor->External_Clock,
                        processor->Core_Count,
                        processor->Core_Enabled,
                        processor->Thread_Count,
                        processor->Voltage,
 			processor->Byte_Order,
 			processor->BogoMIPS ? processor->BogoMIPS : "UnKnown",
 			processor->L1d!=NULL? processor->L1d :"Not Provided",
 			processor->L1i!=NULL? processor->L1i :"Not Provided",
 			processor->L2,
 			processor->L3,
 			processor->mode
                        );
  }
}
